@@ -67,25 +67,31 @@ class CVS(object):
             self.prefix = os.path.join(self.root, self.module)
 
         self.localid = None
-        self.parse_options()
+        self.parse_config()
 
         self.statcache = {}
         self._rcs_log_keyword_re = re.compile('(.*)\$Log(?::[^$\r\n]+)?\$(.*)')
         self._rcs_keyword_re = re.compile('\$([A-Z][A-Za-z]+)(:[^$\r\n]*)?\$')
         self._rcs_strip_attic_re = re.compile('(Attic/)?([^/]+),v$')
 
-    def parse_options(self):
-        """Extract relevant information from the CVSROOT/options file,
+    def parse_config(self):
+        """Extract relevant information from the CVSROOT/config file,
         if it exsits.  At the moment, the only information that is
         used from the file is the custom Id tag keyword 'tag'."""
 
-        filename = os.path.join(self.root, 'CVSROOT', 'options')
+        filename = os.path.join(self.root, 'CVSROOT', 'config')
         if not os.path.isfile(filename):
             return
 
         f = file(filename, 'r')
         try:
             for line in f.readlines():
+                # Skip empty and comment-only lines.
+                if re.match('^\s*(#.*)?$', line) != None:
+                    continue
+
+                # Remaining lines must be "key=value" pairs.  split() will
+                # fail otherwise.
                 option, value = line.split('=', 2)
                 if option == 'tag':
                     self.localid = value.strip()
@@ -265,44 +271,52 @@ class CVS(object):
 
         self.metadb.update_statcache({rcsfile:identity})
 
-    def generate_changesets(self, progress=None, limit=None):
+    def generate_changesets(self, progress=None, limit=None, flush=False):
         """Convert changes stored in the meta database into sets of
         related changes and store the resulting changesets in the meta
         database as well.
+
+        If limit is given, only that many changesets will be imported and
+        all others will be retained.  This can be used to import changesets
+        in small batches.
+
+        If flush is True, all changesets produced by the ChangeSetGenerator
+        are imported; otherwise, potentially incomplete changesets may be
+        retained for the next incremental import.  Use this flag if you can
+        be sure that the CVS repository is consistent and is not going to be
+        modified during the import.
         """
         if progress == None:
             progress = NoProgress()
 
         with progress:
             count = 0
-            csg = ChangeSetGenerator()
+            csg = ChangeSetGenerator(limit=limit)
             total = self.metadb.count_changes()
             progress(_('Processing changes'), 0, total)
+
             for change in self.changes(processed=False, reentrant=True):
                 count += 1
                 progress(_('Processing changes'), count, total)
                 for cs in csg.integrate(change):
-                    # XXX: not reflected in progress output, and ugly
-                    if limit:
-                        if limit > 0:
-                            limit -= 1
-                        else:
-                            return
                     self.metadb.add_changeset(cs)
-            for cs in csg.finalize():
-                # XXX: not reflected in progress output, and ugly
-                if limit:
-                    if limit > 0:
-                        limit -= 1
-                    else:
-                        return
-                self.metadb.add_changeset(cs)
 
-    def fetch(self, progress=None, limit=None):
+        if flush:
+            # All changesets are assumed to be complete and will be
+            # imported.  Note that the ChangeSetGenerator still counts
+            # changesets from flush() against the specified limit.
+            for cs in csg.flush():
+                self.metadb.add_changeset(cs)
+        elif len(csg.changesets) > 0:
+            # The ChangeSetGenerator retained some changesets because
+            # they are potentially incomplete.
+            progress(_('Retained changesets'), len(csg.changesets))
+
+    def fetch(self, progress=None, limit=None, flush=False):
         """Fetch new revisions and compute changesets.
         """
         self.fetch_changes(progress)
-        self.generate_changesets(progress, limit)
+        self.generate_changesets(progress, limit, flush)
 
     def changesets(self):
         """Yield new changesets computed earlier.
